@@ -3,6 +3,9 @@ use v5.26;
 
 package App::url;
 
+our $VERSION = '1.004_01';
+
+use Carp qw(carp);
 use Mojo::Base -strict, -signatures;
 use Mojo::URL;
 use String::Sprintf;
@@ -15,9 +18,12 @@ App::url - format a URL according to a sprintf-like template
 
 =head1 SYNOPSIS
 
-	$ url '%H' http://www.example.com/a/b/c
+	$ url '%h' http://www.example.com/a/b/c
 	www.example.com
 
+	$ url '%H' http://www.example.com/a/b/c
+	www
+
 	$ url '%P' http://www.example.com/a/b/c
 	/a/b/c
 
@@ -32,13 +38,17 @@ Decompose the URL and reformat it according to
 
 =over 4
 
-=item * C<%a> - the path,
+=item * C<%a> - the path
 
 =item * C<%f> - the fragment
 
-=item * C<%h> - the hostname
+=item * C<%h> - the hostname, with domain info
 
-=item * C<%h> - the hostname in punycode
+=item * C<%H> - the hostname without domain info
+
+=item * C<%i> - the hostname in punycode
+
+=item * C<%I> - space-separated list of IP addresses for the host
 
 =item * C<%P> - the password of the userinfo portion
 
@@ -48,6 +58,8 @@ Decompose the URL and reformat it according to
 
 =item * C<%s> - the scheme
 
+=item * C<%S> - the public suffix
+
 =item * C<%u> - the complete URL
 
 =item * C<%U> - the username of the userinfo portion
@@ -87,17 +99,44 @@ You can use this code under the terms of the Artistic License 2.
 
 =cut
 
-our $VERSION = '1.001_01';
+no warnings 'uninitialized';
 
+# $w - width of field
+# $v - value that corresponds to position in template
+# $V - list of all values
+# $l - letter
 my $formatter = String::Sprintf->formatter(
 	a   => sub ( $w, $v, $V, $l ) { $V->[0]->path      },
 	f   => sub ( $w, $v, $V, $l ) { $V->[0]->fragment  },
 	h   => sub ( $w, $v, $V, $l ) { $V->[0]->host      },
+	H   => sub ( $w, $v, $V, $l ) { ( split /\./, $V->[0]->host )[0] },
 	i   => sub ( $w, $v, $V, $l ) { $V->[0]->ihost     },
-	p   => sub ( $w, $v, $V, $l ) { $V->[0]->port      },
+	I   => sub ( $w, $v, $V, $l ) {
+		state $rc = require Socket;
+		my @addresses = gethostbyname( $V->[0]->host );
+		@addresses = map { Socket::inet_ntoa($_) } @addresses[4..$#addresses];
+		"@addresses";
+		},
+	p   => sub ( $w, $v, $V, $l ) { $V->[0]->port // do {
+			if(    $V->[0]->protocol eq 'http'  ) {  80 }
+			elsif( $V->[0]->protocol eq 'https' ) { 443 }
+			};
+	     },
 	P   => sub ( $w, $v, $V, $l ) { $V->[0]->password  },
 	'q' => sub ( $w, $v, $V, $l ) { $V->[0]->query     },
 	's' => sub ( $w, $v, $V, $l ) { $V->[0]->protocol  },
+
+	S   => sub ( $w, $v, $V, $l ) {
+		state $rc = eval { require Net::PublicSuffixList };
+		unless( $rc ) {
+			carp "%${l} requires Net::PublicSuffixList\n";
+			return;
+			}
+		state $psl = Net::PublicSuffixList->new;
+		my $hash = $psl->split_host( $V->[0]->host );
+		$hash->{suffix};
+		},
+
 	U   => sub ( $w, $v, $V, $l ) { $V->[0]->username  },
 	u   => sub ( $w, $v, $V, $l ) { $V->[0]->to_string },
 
